﻿// hmcHilfslinien.jsx
// Copyright © by Manfred Clemens erstellt: 20.05.2013, letzte Änderung: 21.05.2013
// ********************************************************************************
/* 
  PROGRAMMBESCHREIBUNG:
  Das Programm erstellt und löscht Hilfslinien in Photoshop CS

  INSTALLATION IN PHOTOSHOP & SKRIPTAUFRUF:
  Die Datei 'hmcHilfslinien.jsx' in das Verzeichnis
  'c:\Programme\Adobe\Adobe Photoshop CS6\Presets\Scripts' kopieren und Photoshop
  neu starten, danach ein Bilddokument in Photoshop öffnen und den Menübefehl
  'Menü > Datei > Skripten > hmcHilfslinien' aufrufen und anwenden.
*/  
#target Photoshop
#strict on

// Benutzer Einstellungen
var scriptName = "hmcHilfslinien";
var scriptVersion = " 1.0";


try {
  
  function clearAllGuides(){
    // Alle vorhandenen Hilfslinien löschen
    var id104 = charIDToTypeID( "Dlt " );
    var desc23 = new ActionDescriptor();
    var id105 = charIDToTypeID( "null" );
    var ref7 = new ActionReference();
    var id106 = charIDToTypeID( "Gd  " );
    var id107 = charIDToTypeID( "Ordn" );
    var id108 = charIDToTypeID( "Al  " );
    ref7.putEnumerated( id106, id107, id108 );
    desc23.putReference( id105, ref7 );
    executeAction( id104, desc23, DialogModes.NO );
  }
  
  function createGuide(pos, orientation){
    // Neue Hilfslinie erzeugen
    var id18 = charIDToTypeID( "Mk  " );
	  var desc3 = new ActionDescriptor();
	  var id19 = charIDToTypeID( "Nw  " );
	  var desc4 = new ActionDescriptor();
	  var id20 = charIDToTypeID( "Pstn" );
	  var id21 = charIDToTypeID( "#Pxl" );
	  desc4.putUnitDouble( id20, id21, pos );
	  var id22 = charIDToTypeID( "Ornt" );
	  var id23 = charIDToTypeID( "Ornt" );
	  var id24 = charIDToTypeID( orientation );
	  desc4.putEnumerated( id22, id23, id24 );
	  var id25 = charIDToTypeID( "Gd  " );
	  desc3.putObject( id19, id25, desc4 );
    executeAction( id18, desc3, DialogModes.NO );
	}

  // Check ob numerische Zahleneingabe
  function IsNumeric(sText) {
    var ValidChars = "0123456789";
    var IsNumber = true;
    var Char;
    for (i = 0; i < sText.length && IsNumber == true; i++) { 
      Char = sText.charAt(i); 
      if (ValidChars.indexOf(Char) == -1) {
        IsNumber = false;
        alert("Keine gültige numerische Zahleneingabe!", "Fehler", Error);
      }
    }
    return IsNumber;
  }
  

  
  function CreateDialog() { 
    this.windowRef = null; 

    // **************************************************************************
    // ***                        Dialog hinzufügen                           ***
    // **************************************************************************

    var dlg = new Window("dialog", scriptName + scriptVersion ,[100,100,380,365]); 
    this.windowRef = dlg;
  
    // **************************************************************************
    // ***                        Panel hinzufügen                            ***
    // **************************************************************************
    dlg.infoPanel = dlg.add("panel", [10,10,270,150], ""); 
    with (dlg.infoPanel) {
      infoFileName = add('statictext',[10,10,260,25], 
                          "Dateinname: " + app.activeDocument.name);
                          
      chbClearAllGuides = add('checkbox', [10,35,250,50], 
                          "Alle vorhandenen Hilfslinien löschen");
      chbClearAllGuides.value = true;
      
      chbCreateMidGuides = add('checkbox', [10,55,250,70], 
                          "Mittellinien");
      chbCreateMidGuides.value = true;
      
      chbCreateBorderGuides = add('checkbox', [10,75,190,90], 
                          "Randlinien, Randabstand");
      txtBorderDistance = add('edittext', [200,75,230,92], "10");
      txtBorderDistance1 = add('statictext', [235,75,260,90], "%");
                          
      chbGridGuides = add('checkbox', [10,95,190,110], 
                          "Raster, Anzahl der Teilungen");
      txtGridGuides = add('edittext', [200,95,230,112], "5");                    
                          
    }

    // **************************************************************************
    // ***                        Panel hinzufügen                            ***
    // **************************************************************************
    dlg.btnPanel = dlg.add("panel", [10,200,270,250], "");
    with (dlg.btnPanel) {
      // Button's hinzufügen
      okBtn = dlg.btnPanel.add("button", [10,10,110,35], "OK");
      cancelBtn = dlg.btnPanel.add("button", [145,10,245,35], "Abbruch");

      // ************************************************************************
      // ***                  Klick-Ereignise auswerten                       ***
      // ************************************************************************
      // OK-Button
      okBtn.onClick = function() {
        
        // Originale Bildabmessungen (Pixel) in Variablen speichern
        var docWidth = app.activeDocument.width.value;
        var docHeight = app.activeDocument.height.value;
        
        // Alle vorhandenen Hilfslinien löschen
        if (chbClearAllGuides.value == true) {
          clearAllGuides();
        }
      
        // Mittellinien hinzufügen
        if (chbCreateMidGuides.value == true) {
          createGuide(docHeight/2, "Hrzn" );
          createGuide(docWidth/2, "Vrtc" );
        }
        
        // Randlinien hinzufügen
        if (chbCreateBorderGuides.value == true) {
          // Eingabewerte überprüfen
          if (IsNumeric(txtBorderDistance.text)) {
            // Die Randbreite wird immer von der kürzeren Bildseite berechnet  
            if (docWidth >= docHeight) {
              var borderThickness = Math.round(docHeight * txtBorderDistance.text / 100);
            }  
            else  {
              var borderThickness = Math.round(docWidth * txtBorderDistance.text / 100);
            }  
            // vertikale Linien
            createGuide(borderThickness, "Vrtc" );
            createGuide(docWidth - borderThickness, "Vrtc" );
            // horizontale Linien
            createGuide(borderThickness, "Hrzn" );
            createGuide(docHeight - borderThickness, "Hrzn" );
          }
        }
      
        // Raster hinzufügen
        if (chbGridGuides.value == true) {
          // Eingabewerte überprüfen
          if (IsNumeric(txtGridGuides.text)) {
            var n = txtGridGuides.text;
            
            // vertikale Linien
            var d = Math.round(docWidth / n);
            var x = 0;
            for (var i = 0; i <= n; i ++) {
              createGuide(x, "Vrtc" );
              x += d;
            } 
             
            // horizontale Linien
            var d = Math.round(docHeight / n);
            var x = 0;
            for (var i = 0; i <= n; i ++) {
              createGuide(x, "Hrzn" );
              x += d;
            } 
          }  
        }  
        
        dlg.close();
      }
    
      // Abbruch-Button
      cancelBtn.onClick = function() {
        dlg.close();
      }
    }
   
    // **************************************************************************
    // ***                 Dialog anzeigen und plazieren                      ***
    // **************************************************************************
    //dlg.center();
    dlg.frameLocation = [600,100];              // Dialog-Fenster plazieren
    dlg.show();                                 // Dialog-Fenster anzeigen
    //return true;

    // Original PS-Einstellungen wieder zurücksetzen
    function Cleanup() {
      app.preferences.rulerUnits = defaultRulerUnits;
      app.preferences.typeUnits = defaultTypeUnits;
      app.DisplayDialogs = defaultDisplayDialogs;
      return;
    }
}

// ******************************************************************************
// ***                    Hauptprogramm - Programmstart                       ***
// ******************************************************************************
// Ist überhaupt ein Dokument geöffnet?
if (documents.length > 0) {
    try {
        app.bringToFront(); // Zunächst wird Photoshop in den Vordergrund geholt
        // Original Einstellungen speichern und die gewünschte Maßeinheit setzen        
        var defaultRulerUnits = app.preferences.rulerUnits;
        var defaultTypeUnits = app.preferences.typeUnits;
        var defaultDisplayDialogs = app.DisplayDialogs;
        if (defaultRulerUnits != Units.PIXELS) {
          preferences.rulerUnits = Units.PIXELS; 
        }
        CreateDialog(); // Hauptprogramm - Programmstart
        
    } catch(e) {        // Fehler anzeigen
        if (e.line != undefined)
            alert(scriptName + " Zeile[" + e.line + "] " + e);
        else
            alert(scriptName + " " + e);
        Cleanup();    
        }
} else {
    alert(scriptName + " Es ist kein Bilddokument geöffnet!");
}

// Speicher wieder freimachen
scriptName = null;
scriptVersion = null;


} // try Ende
catch (e) {
  alert (e.description, "Fehler!");
} // EOF